; SerBrd.ASM           Serial I/O board based on AT90S8535
;============================================================
; This software is written for an AT90S8535 which is used
; to implement digital I/O and analog input capability.
; It communicates with the host system via a UART serial
; connection.  The baud rate can be selected with jumpers.
;   (Available Baud Rates:  2400, 4800, 9600, 19200.)
; The system can be set up for basic I/O as well as events
; at the bit level or byte level.
; An 8.000MHz crystal is assumed for the AT90S8535.
;==========================================================
.equ	PORTA           = $1b
.equ	DDRA            = $1a
.equ	PINA            = $19
.equ	PORTB           = $18
.equ	DDRB            = $17
.equ	PINB            = $16
.equ	PORTC           = $15
.equ	DDRC            = $14
.equ	PINC            = $13
.equ    PORTD           = $12      ;PORTD, DDRD, and PIND
.equ    DDRD            = $11      ;are used setup and control
.equ    PIND            = $10      ;(not user-accessible)

.equ  UDR             = $0c
.equ  USR             = $0b
.equ  UBRR            = $09
.equ  UCR             = $0a
.equ  ADMUX           = $07
.equ  ADCSR           = $06
.equ  ADCH            = $05
.equ  ADCL            = $04

.equ  RAMEND          = $25F        ;end of SRAM in the micro
.equ  BUFFER_START    = $80         ;start of the storage buffer. 
.equ  BUFFER_LENGTH   = $20         ;nominal length of storage buffer

.equ  SPL             = $3D
.equ  SPH             = $3E
.equ  SREG            = $3F

.def  LF_FLAG  = r12                ;do or don't send LF after CR
.def  P1_BYTE_MASK    = r13         ;byte event indicator for Port 1
.def  P2_BYTE_MASK    = r14         ;byte event indicator for Port 2
.def  P3_BYTE_MASK    = r15         ;byte event indicator for Port 3

.def  P1_DATA         = r20         ;events, data reg. for Port 1
.def  P1_BIT_MASK     = r21         ;bit event mask reg. for Port 1
.def  P2_DATA         = r22         ;events, data reg. for Port 2 
.def  P2_BIT_MASK     = r23         ;bit event mask reg. for Port 2
.def  P3_DATA         = r24         ;events, data reg. for Port 3
.def  P3_BIT_MASK     = r25         ;bit event mask reg. for Port 3

.def  XL              = r26         ;use for storing the incoming characters        
.def  XH              = r27         ;in the buffer.
.def  NUM_OF_CMDS     = r28         ;number of commands in the buffer
.def  ZL              = r30         ;use for retrieving commands and data
.def  ZH              = r31         ;from the buffer.

;========================================================
.DSEG                                     ;data segment
        .org BUFFER_START
        BUFFER: .BYTE BUFFER_START+BUFFER_LENGTH+15         ;set aside space for buffer


;==========================================================
.CSEG                                     ;code segment

;---------------------Interrupt Vectors--------------------
        .org 0                            ;start of program execution after reset
        rjmp starthere

        .org $B                           ;address for UART receive interrupt
        rjmp UART_RCV                     ;go to subroutine to receive command string

;----------------------Main Program Starts Here------------
        .org $10 
starthere:                                ;INITIALIZATION AND SETUP ROUTINE
        ldi r16, high(RAMEND)             ;set up the stack pointer
        out SPH, r16		
        ldi r16, low(RAMEND)
        out SPL, r16	

        ldi r16, 0                        ;reset all i/o ports
        out DDRA, r16
        out PORTA, r16
        out DDRB, r16
        out PORTB, r16
        out DDRC, r16
        out PORTC, r16

        ldi r16, 0x54
        out DDRD, r16                     ;set up as 5 inputs, 3 outputs for now
        out PORTD, r16                    ;disable the buffers for now

        rcall UART_Setup                  ;UART initialization and buffer setup
        ldi NUM_OF_CMDS , 0               ;initialize NUM_OF_CMDS
        
                 ;set up the data regs and event regs. -- clear them all
        ldi P1_DATA, 0                    ;events, data reg. for Port 1
        ldi P1_BIT_MASK, 0                ;events, bit event mask reg. for Port 1
        ldi P2_DATA, 0                    ;events, data reg. for Port 2
        ldi P2_BIT_MASK, 0                ;events, bit event mask reg. for Port 2
        ldi P3_DATA, 0                    ;events, data reg. for Port 3
        ldi P3_BIT_MASK, 0                ;events, bit event mask reg. for Port 3
        mov P1_BYTE_MASK, P1_DATA         ;byte events enabled indicator for A
        mov P2_BYTE_MASK, P1_DATA         ;byte events enabled indicator for B
        mov P3_BYTE_MASK, P1_DATA         ;byte events enabled indicator for C
        mov LF_FLAG, P1_DATA              ;clear LF_FLAG = no LF after CR

        ldi XH, high(BUFFER_START)
        ldi XL, low(BUFFER_START)         ;initialize storage pointer
        ldi ZH, high(BUFFER_START)
        ldi ZL, low(BUFFER_START)         ;initialize retrieve pointer

        ldi r16, $98                      ;enable TX, RX, and RX interrupts
        out UCR, r16

        ldi r16, $80
        out SREG, r16                     ;set global interrupt enable bit


;=============================================
; ***   START OF THE MAIN CONTROL LOOP   ***
; The main loop does event-checking on
; ports 1, 2, and 3 and calls subroutines
; as appropriate.  After events have been
; dealt with, the next command in the buffer
; will be processed and executed.
;=============================================

ChkByteEv1:
        mov r16, P1_BYTE_MASK             ;Get event byte indicator for port 1.
        cpi r16, 0                        ;See if byte events are enabled for Port 1.
        breq ChkBitEv1                    ;If byte events are not enabled, then check bit ev.        
        in r16, PINA                      ;get new Port 1 data, put in r16
        cp r16, P1_DATA                   ;compare old data with new data
        breq ChkByteEv2                   ;if the data is the same, then move on to Port 2
        mov P1_DATA, r16                  ;else, if diff., then save the new data in, P1_DATA 
        rcall ByteEv1                     ;and call sub for byte events on Port 1
        rjmp ChkByteEv2                   ;then move on to Port 2

ChkBitEv1:
        cpi P1_BIT_MASK, 0                ;See if bit events for Port 1 are enabled.
        breq ChkByteEv2                   ;If not, then move on to Port 2.
chke11: mov r17, P1_DATA                  ;save old Port 1 event data in r17
        in r16, PINA                      ;get new Port 1 data
        mov r18, r16                      ;save new Port 1 data in r18
        and r16, P1_BIT_MASK              ;AND new data with the event mask
        and P1_DATA, P1_BIT_MASK          ;AND old data with the event mask
        eor P1_DATA, r16                  ;XOR masked data registers
        cpi P1_DATA, 0                    ;see if they are the same; if so, then no event occurred
        brne AA                           ;if != 0, then there was an event
        mov P1_DATA, r17                  ;else, no event... restore the old data
        rjmp ChkByteEv2                   ;and continue on
AA:     rcall BitEv1                      ;call bit ev. sub, return with old event data in r17
        mov P1_DATA, r17                  ;save the new data in Port 1 event data reg, P1_DATA	
        rjmp chke11                       ;check for another event until there are none
	
ChkByteEv2:
        mov r16, P2_BYTE_MASK             ;Get Port 2 byte event indicator.
        cpi r16, 0                        ;See if byte events are enabled for Port 2.
        breq ChkBitEv2                    ;If not, then check for bit events.
        in r16, PINB                      ;byte events enabled, get Port 2 data
        cp r16, P2_DATA                   ;compare old data with new data
        breq ChkByteEv3                   ;if the data is the same, then move on to Port 3
        mov P2_DATA, r16                  ;else, if diff., then save new data in P2_DATA
        rcall ByteEv2                     ;and call sub for byte events on Port 2
        rjmp ChkByteEv3                   ;then move on to Port 3

ChkBitEv2:
        cpi P2_BIT_MASK, 0                ;See if bit events for Port 2 are enabled.
        breq ChkByteEv3                   ;If not, then move on to Port 3.
chke22: mov r17, P2_DATA                  ;save Port 2 event data in r17
        in r16, PINB                      ;get new Port 2 data
        mov r18, r16                      ;save new Port 2 data in r18
        and r16, P2_BIT_MASK              ;AND new data with the event mask
        and P2_DATA, P2_BIT_MASK          ;AND old data with the event mask
        eor P2_DATA, r16                  ;XOR masked data registers
        cpi P2_DATA, 0                    ;see if they are the same; if so, then no event occurred
        brne BB                           ;if not equal to 0, then there was an event
        mov P2_DATA, r17                  ;else, no event... restore the old data
        rjmp ChkByteEv3                   ;and continue on
BB:     rcall BitEv2                      ;if event, then call sub for bit events on Port B.
        mov P2_DATA, r17                  ;save the new data in Port 2 event data reg, P2_DATA		
        rjmp chke22                       ;check for another event until there are none

ChkByteEv3:
        mov r16, P3_BYTE_MASK             ;Get Port 3 byte event indicator.
        cpi r16, 0                        ;See if byte events are enabled for Port 3.
        breq ChkBitEv3                    ;If not, then check for bit events.
        in r16, PINC                      ;byte events enabled, get Port 3 data
        cp r16, P3_DATA                   ;compare old data with new data
        breq chkev4                       ;if the data is the same, then move on to next..
        mov P3_DATA, r16                  ;else, if diff., then save new data in P3_DATA and
        rcall ByteEv3                     ;call sub for byte events on Port 3.
        rjmp chkev4                       ;and move on...

ChkBitEv3:
        cpi P3_BIT_MASK, 0                ;See if the bit events for Port 3 are enabled.
        breq chkev4                       ;If not, then we're done checking for now.
chke33: mov r17, P3_DATA                  ;save Port 3 event data in r17
        in r16, PINC                      ;get new Port 3 data
        mov r18, r16                      ;save new Port 3 data in r18
        and r16, P3_BIT_MASK              ;AND new data with the event mask
        and P3_DATA, P3_BIT_MASK          ;AND old data with the event mask
        eor P3_DATA, r16                  ;XOR masked data registers
        cpi P3_DATA, 0                    ;see if they are the same; if so, then no event occurred
        brne CC                           ;if not equal then event: call the eventC sub 
        mov P3_DATA, r17                  ;else, no event... restore the old data	
        rjmp chkev4                       ;and continue on
CC:     rcall BitEv3                      ;if event, then call sub for bit events on Port 3.
        mov P3_DATA, r17                  ;save the new data in Port 3 event data reg, P3_DATA
        rjmp chke33                       ;check for another event until there are none

chkev4:
        cpi NUM_OF_CMDS, 0                ;see if there are any commands waiting
        breq No_Cmds                      ;if none, then don't process
        rcall PROCESS_BUFFER              ;else, execute the next command
No_Cmds:
        rjmp ChkByteEv1                   ;keep looping back

;========================================================
; UART_RCV is only called due to serial character being
; received and causing an interrupt.  Each incoming char-
; acter is stored in the buffer and processed later.  If
; a CR is received, then the NUM_OF_CMDS is incremented.
; The buffer wraps around after a CR when it gets too long.
;========================================================
UART_RCV:
        push r17                          ;save r17
        push r16                          ;save r16

        in r17, SREG                      ;get the status register...
        push r17                          ;...and save it on the stack

        in r17, UDR                       ;get character from UART
        cpi r17, 0                        ;see if the received char. is NULL
        breq No_CR                        ;if NULL then ignore, don't store
        cpi r17, 10                       ;see if the received char. is LF
        breq No_CR                        ;if LineFeed then ignore, don't store

        st X+, r17                        ;store the character and increment pointer
        cpi r17, 13                       ;see if a CR was received
        brne No_CR                        ;if not, then branch

        inc NUM_OF_CMDS                   ;else, increment # of commands waiting

        ldi r17, BUFFER_START+BUFFER_LENGTH
        cp XL, r17                        ;see we've gone past nominal buffer length
        brlo No_CR                        ;if XL<end_of_buffer, no rollover needed
        ldi XL, BUFFER_START              ;else, go back to the start
No_CR:

        pop r17                           ;get status register from the stack...
        out SREG, r17                     ;...and restore status register

        pop r16                           ;restore r16
        pop r17                           ;restore r17
        reti

;========================================================
; PROCESS_BUFFER reads the next command from the buffer
; and executes it.
;========================================================
PROCESS_BUFFER:
        ld r16, Z                         ;get the next character to process

                                          ;Read a byte from a port?
t1:     cpi r16, 'I'                      ;see if first character is 'I'
        brne t2                           ;if not 'I', then check for the next condition
        rcall read                        ;if 'I', then call the read subroutine
        rjmp ldone

                                          ;Read a bit from a port pin?
t2:     cpi r16, 'i'                      ;see if first character is 'i'
        brne t3                           ;if not 'i', then check for the next condition
        rcall readbit                     ;if 'i', then call the readbit sub
        rjmp ldone

                                          ;Set up a port for input or output?
t3:     cpi r16, 'S'                      ;see if first char. is 'S'
        brne t4                           ;if not 'S', then check for the next condition
        rcall setdir                      ;if 'S', then call the setdir subroutine
        rjmp ldone

                                          ;Output a byte on a port?
t4:     cpi r16, 'O'                      ;see if first char. is 'O'
        brne t5                           ;if not 'O', then check for next condition
        rcall outp                        ;if 'O', then call the outp subroutine
        rjmp ldone	

                                          ;Output a bit on a port pin?
t5:     cpi r16, 'o'                      ;see if first char. is 'o'
        brne t6                           ;if not 'o', then check for next condition
        rcall outpbit                     ;if 'o', then call the outpbit sub
        rjmp ldone
	
                                          ;Enable byte events on a port?
t6:     cpi r16, 'V'                      ;see if first char. is 'V'
        brne t7                           ;if not 'V', then check for next condition
        rcall event                       ;if 'V', then call the event sub to set up for byte events
        rjmp ldone

                                          ;Enable bit events on a port pin?
t7:     cpi r16, 'v'                      ;see if first char is 'v'
        brne t8                           ;if not 'v', then check for next condition
        rcall eventbit                    ;if 'v', then call the event sub to set up for bit events
        rjmp ldone

                                          ;Clear byte events on a port?
t8:     cpi r16, 'C'                      ;see if first char is 'C'
        brne t9                           ;if not 'C', then check for next condition
        rcall clrev                       ;if 'C', then call the clrev sub to clear byte event
        rjmp ldone

                                          ;Clear bit events on a port pin?
t9:     cpi r16, 'c'                      ;see if first char is 'c'
        brne t10                          ;if not 'c', then check for next condition
        rcall clrevbit                    ;if 'c', then call the clrevbit sub to clear bit event
        rjmp ldone

                                          ;Enable the buffer for a port?
t10:    cpi r16, 'E'                      ;see if first char is 'E'
        brne t11                          ;if not 'E', then check for next condition
        rcall enport                      ;if 'E', then call enport sub to enable a port
        rjmp ldone

                                          ;Disable the buffer for a port?
t11:    cpi r16, 'X'                      ;see if first char is 'X'
        brne t12                          ;if not 'X', then check for next condition
        rcall disport                     ;if 'X', then call disport to disable a port
        rjmp ldone

                                          ;Ping command, send back the character  'G'?
t12:    cpi r16, 'P'                      ;see if first char is 'P'
        brne t13                          ;if not 'P', then check next
        rcall ping                        ;if 'P', then call ping sub
        rjmp ldone

                                          ;Reset the serial board?
t13:    cpi r16, 'R'                      ;see if first char is 'R'
        brne t14                          ;if not 'R', then check next
        rjmp starthere                    ;go back to the beginning to start over, clearing events and 
                                          ;setting up ports as inputs, enabled
        rjmp ldone		

                                          ;set up for analog inputs?
t14:    cpi r16, 'A'                      ;see if first char is 'A'
        brne t15                          ;if not 'A', then check next
        rcall SetAnalog                   ;call sub to set up port for analog input
        rjmp ldone      

                                          ;set up for digital i/o?
t15:    cpi r16, 'D'                      ;see if first char is 'D'
        brne t16                          ;if not 'D', then check next
        rcall SetDig                      ;call sub to set up port for digital i/o
        rjmp ldone

                                          ;take an analog reading?                                                
t16:    cpi r16, 'a'                      ;see if first char is 'a'
        brne t17                          ;if not 'a', then check next
        rcall GetAnalog                   ;call sub to get an analog reading
        rjmp ldone

t17:    cpi r16, 'L'                      ;toggle line feeds (after CR)?
        brne t18                          ;if not 'L' then check next
        rcall ToggleLF                    ;call the sub to toggle (LF are off after reset)
        rjmp ldone

t18:    rcall error                       ;else, error condition!

ldone:
        rcall FIND_NEXT_CMD               ;point Z to start of next command
        dec NUM_OF_CMDS                   ;one less command needs to be done

        ret                               ;return from interrupt routine

;=====================================================================
; Set the direction for a port (input or output)
;   Command format:  S1,1
;   Upon entry, Z points to the first character in the string.
;=====================================================================
setdir:
        ldd r16, Z+1                      ;get the second character, port number ('1' to '3')

try11:	cpi r16, '1'                  ;see if second char is '1'
        brne try12                        ;if not '1', then see if 2
        ldd r16, Z+3                      ;else, it is '1', so point to the data char
        cpi r16, 48                       ;see if the data char is "0"
        breq inp1                         ;if it is zero, then port1 needs to be input port
outp1:  in r16, PORTD                     ;get PORTD data
        sbr r16, 8                        ;set bit 3 in r16
        out PORTD, r16                    ;so that PORT 1 buffer is configured as output
        ldi r16, $ff                      ;load 255 into r16
        out DDRA, r16                     ;set port 1 for output
        rjmp outdone2                     ;done

inp1:   ldi r16, 0                        ;load 0 into r16
        out DDRA, r16                     ;set port 1 for input
        in r16, PORTD                     ;get PORTD data
        cbr r16, 8                        ;clear bit 3 in r16
        out PORTD, r16                    ;so that PORT 1 buffer is configured as input
        rjmp outdone2                     ;done

try12:  cpi r16, '2'                      ;see if second char is '2'
        brne try13                        ;if not '2', then see if 3
        ldd r16, Z+3                      ;else, it is '2', so get the data char
        cpi r16, 48                       ;see if the data char is "0"
        breq inp2                         ;if it is zero, then port2 needs to be input port
outp2:  in r16, PORTD                     ;get PORTD data
        sbr r16, 32                       ;set bit 5 in r16
        out PORTD, r16                    ;so that PORT 2 buffer is configured as output
        ldi r16, $ff                      ;load 255 into r16
        out DDRB, r16                     ;set port 2 for output
        rjmp outdone2                     ;done

inp2:	  ldi r16, 0                        ;load 0 into r16
        out DDRB, r16                     ;set port 2 for input
        in r16, PORTD                     ;get PORTD data
        cbr r16, 32                       ;clear bit 5 in r16
        out PORTD, r16                    ;so that PORT 2 buffer is configured as input
        rjmp outdone2                     ;done

try13:  cpi r16, '3'                      ;see if second char is '3'
        brne try14                        ;if not '3', then see if 4
        ldd r16, Z+3                      ;else, it is '3', so get the data char
        cpi r16, 48                       ;see if the data char is "0"
        breq inp3                         ;if it is zero, then port2 needs to be input port
outp3:  in r16, PORTD                     ;get PORTD data
        sbr r16, 128                      ;set bit 7 in r16
        out PORTD, r16                    ;so that PORT 3 buffer is configured as output
        ldi r16, $ff                      ;load 255 into r16
        out DDRC, r16                     ;set port 3 for output
        rjmp outdone2                     ;done

inp3:   ldi r16, 0                        ;load 0 into r16
        out DDRC, r16                     ;set port 3 for input
        in r16, PORTD                     ;get PORTD data
        cbr r16, 128                      ;clear bit 7 in r16
        out PORTD, r16                    ;so that PORT 3 buffer is configured as input
        rjmp outdone2                     ;done


try14:  rcall error                       ;error condition

outdone2:

	ret                                 ;go all the way back
		

;===================================================================			
; Error sub sends out an exclamation point
;===================================================================
error:  ldi r29, '!'
        rcall xmit
        ldi r29, 13
        rcall xmit
        ret


;===================================================================	 	
; Read the data on a port.
;  Command format:  I2
;  Upon entry, Z points to 1st character in command string.
;===================================================================
read:        
        ldd r16, Z+1                     	;get the second character (port #)
try31:  cpi r16, '1'                      ;see if second char is '1'
        brne try32                        ;if not '1', then see if 2
        rjmp read1                        ;done outputting data
try32:  cpi r16, '2'                      ;see if second char is '2'
        brne try33                        ;if not '2', then see if 3
        rjmp read2                        ;done outputting data
try33:  cpi r16, '3'                      ;see if second char is '3'
        brne try34                        ;if not '3', then see if ...
        rjmp read3
try34:  rcall error                       ;error condition
        rjmp readdone

read1:  in r16, PINA                      ;send "P1=" for Port 1 reads
        ldi r29, 'P'
        rcall xmit
        ldi r29, '1'
        rcall xmit
        ldi r29, '='
        rcall xmit
        rjmp calcread
read2:  in r16, PINB                      ;send "P2=" for Port 2 reads
        ldi r29, 'P'
        rcall xmit
        ldi r29, '2'
        rcall xmit
        ldi r29, '='
        rcall xmit
        rjmp calcread
read3:  in r16, PINC                      ;send "P3=" for Port 3 reads
        ldi r29, 'P'
        rcall xmit
        ldi r29, '3'
        rcall xmit
        ldi r29, '='
        rcall xmit
        rjmp calcread

calcread:                                 ;calculate the ascii equivalent of data in r16
                                          ;and return (in hexadecimal format)
        mov r17, r16                      ;get data in r17
        andi r17, $f0                     ;clear the low nibble
        swap r17                          ;swap upper and lower nibbles
        rcall ConvR17                     ;convert the r17 to a hex char and return in r29
        rcall xmit                        ;send out the character
        mov r17, r16                      ;get data in r17
        andi r17, $0f                     ;clear the upper nibble
        rcall ConvR17                     ;convert r17 to a hex char and return in r29
        rcall xmit
        ldi r29, 13                       ;load carriage return
        rcall xmit                        ;send carriage return

readdone:
        ret

;========================================================================	
; Read a bit on a port.
;   command format: i3.7
;                     
;========================================================================
readbit:                ;upon entry, Z points to the first character
        push r17                          ;save r17 and r18
        push r18

        ldd r16, Z+3                      ;put the bit number in r16
        mov r18, r16                      ;store bit number in r18
        subi r16, 48                      ;convert the ASCII number to binary (o to 7), store in r16
        ldi r17, 1                        ;put 1 in r17, to be shifted to bit position
rbsh:   cpi r16, 0                        ;see if r16 is 0 yet
        breq rbshd                        ;if so, then we're done shifting
        lsl r17                           ;logical shift left on r17
        dec r16                           ;decrement r16
        rjmp rbsh                         ;go back until r16=0	

rbshd:                                    ;r17 now contains a 1 in the place of the bit to be read
        ldd r16, Z+1                      ;get the port number (1 to 3)

RBA:    cpi r16, '1'                      ;see if it's Port 1
        brne RBB                          ;if not, see if it's Port 2
        in r16, PINA                      ;if so, read data on Port 1
        ldi r29, 'p'                      ;send "p1.n=" where n=bit number
        rcall xmit
        ldi r29, '1'
        rcall xmit
        ldi r29, '.'
        rcall xmit
        mov r29, r18                      ;bit number
        rcall xmit
        ldi r29, '='
        rcall xmit
        rjmp readit				;jump to process the information

RBB:    cpi r16, '2'                      ;see if it's Port 2
        brne RBC                          ;if not, see if it's Port 3
        in r16, PINB                      ;if so, then read data on Port 2
        ldi r29, 'p'                      
        rcall xmit
        ldi r29, '2'
        rcall xmit
        ldi r29, '.'
        rcall xmit
        mov r29, r18                      ;bit number
        rcall xmit
        ldi r29, '='
        rcall xmit
        rjmp readit                       ;jump to process the information

RBC:    cpi r16, '3'                      ;see if it's Port 3
        brne RBD                          ;if not, then we're out of ports at this point
        in r16, PINC                      ;if so, then read data on Port 3
        ldi r29, 'p'
        rcall xmit
        ldi r29, '3'
        rcall xmit
        ldi r29, '.'
        rcall xmit
        mov r29, r18                      ;bit number
        rcall xmit
        ldi r29, '='
        rcall xmit
        rjmp readit                       ;jump to process the information

RBD:    rcall error                       ;error condition
        rjmp rbdone

readit: AND r16, r17                      ;AND the input data and the mask having a 1 in the bit position
        cpi r16, 0                        ;see if the result is 0 
        breq rbzero                       ;if so, then the bit is a 0
rbone:  ldi r29, '1'                      ;else, if not zero, then the bit is a 1
        rcall xmit                        ;send '1'
        ldi r29, 13                       ;send a CR
        rcall xmit	
        rjmp rbdone

rbzero: ldi r29, '0'                      ;bit is a 0
        rcall xmit                        ;send '0'
        ldi r29, 13                       ;send a CR
        rcall xmit	
        rjmp rbdone
	
rbdone:

        pop r18                           ;restore r18 and r17
        pop r17
        ret

;=========================================================================
;outp sub converts the hex ASCII data to binary and sends out a port
;  Command format:  O3=B5
;  Z points to the 1st character in the command string.
;=========================================================================
outp:
        push r17                          ;store r17

        ldd r19, Z+3                      ;get the first data character 
        ldd r17, Z+4                      ;get the second character
        cpi r17, 13                       ;see if 2nd character is a carriage return
        breq outplen1                     ;if so, then length of data is 1, so branch there

        ldd r16, Z+5                      ;else, get the next character
        cpi r16, 13                       ;see if the character is a carriage return
        breq outplen2                     ;if so, then length of data is 2, so branch there
        rjmp outperror                    ;else, data is longer than 2 so it's an error; all done

outplen1:                                 ;data character is in r19
        rcall getdata                     ;getdata will convert the character in r19 into a number
                                          ;from 0 - 15 (in binary) and return it in r19
                                          ;on error condition, it will return 255
        cpi r19, 255                      ;see if there's an error
        breq outperror                    ;if there's an error, then jump  
        rjmp calcdone                     ;else, no error, so put out data

outplen2:                                 ;first data char. is in r19
        rcall getdata                     ;getdata will convert the first data char.
                                          ;into a number from 0 to 15 & return in r19
                                          ;on error, it returns 255
        cpi r19, 255                      ;see if there's an error
        breq outperror                    ;if so, then jump
        swap r19                          ;else, move the 1st char. number into
                                          ;the high nibble
        mov r16, r19                      ;and put in r16
        mov r19, r17                      ;put 2nd char in r19
        rcall getdata                     ;and convert to a number
        cpi r19, 255                      ;check for error
        breq outperror                    ;if so, call error
        or r19, r16                       ;else, combine 1st and 2nd char. numbers
                                          ;r19 now contains the data to put out
calcdone:
        ldd r16, Z+1                      ;get the second character
try1:   cpi r16, '1'                      ;see if second char is '1'
        brne try2                         ;if not '1', then see if 2
        out PORTA, r19                    ;else, if it is '1', then output data on Port 1
        rjmp outdone                      ;done outputting data
try2:   cpi r16, '2'                      ;see if second char is '2'
        brne try3                         ;if not '2', then see if 3
        out PORTB, r19                    ;else, if it is '2', then output data on Port 2
        rjmp outdone                      ;done outputting data
try3:   cpi r16, '3'                      ;see if second char is '3'
        brne try4                         ;if not '3', then see if ...
        out PORTC, r19                    ;else, if it is '3', then output data on Port 3
        rjmp outdone
try4:   rjmp outperror                    ;error condition

outperror:
        rcall error
outdone:
        pop r17
        ret                               ;go all the way back

;=========================================================================
; output a bit on a port.
;   Command Format:  o3.1=1
;=========================================================================
outpbit:
        push r17
		
        ldd r16, Z+3                      ;get the bit number in r16
        subi r16, 48                      ;convert the ASCII number to binary (o to 7), store in r16
        ldi r17, 1                        ;put 1 in r17 to be shifted to bit pos.
shift:  cpi r16, 0                        ;see if r16 is 0 yet
        breq shdone                       ;if so, then we're done shifting
        lsl r17                           ;logical shift left on r17
        dec r16                           ;decrement r16
        rjmp shift                        ;go back until r16=0	
shdone:                                   ;r17 now contains a 1 in the place of the bit to be cleared or set
        ldd r19, Z+5                      ;get the data to put out ('0' or '1')
        cpi r19, 48                       ;see if we're clearing a bit: 0
        breq clearbit                     ;if so, jump to the clearbit portion
        cpi r19, 49                       ;see if we're setting a bit: 1
        breq setbit                       ;if so, jump to the setbit portion
        rcall error                       ;else, error condition
        rjmp opbdone	

clearbit:
        com r17                           ;invert so r17 contains 1's except for the bit to be cleared
        ldd r16, Z+1                      ;get the second character (port number)
	
opbc1:  cpi r16, '1'                      ;see if second char is '1'
        brne opbc2                        ;if not '1', then see if 2
        in r16, PORTA                     ;get the data in the PORTA register
        and r16, r17                      ;AND r16 and r17 to clear the selected bit
        out PORTA, r16                    ;send out the new data to Port 1
        rjmp opbdone                      ;done outputting data
opbc2:  cpi r16, '2'                      ;see if second char is '2'
        brne opbc3                        ;if not '2', then see if 3
        in r16, PORTB                     ;get the data in the PORTB register
        and r16, r17                      ;AND r16 and r17 to clear the selected bit
        out PORTB, r16                    ;send out the new data to Port 1
        rjmp opbdone                      ;done outputting data
opbc3:  cpi r16, '3'                      ;see if second char is '3'
        brne opbc4                        ;if not '3', then see if ...
        in r16, PORTC                     ;get the data in the PORTC register
        and r16, r17                      ;AND r16 and r17 to clear the selected bit
        out PORTC, r16                    ;send out the new data to Port 1
        rjmp opbdone
opbc4:  rcall error                       ;error condition
        rjmp opbdone                      ;ready to go back after error condition

setbit:
        ldd r16, Z+1                      ;get the second character (port number)
	
opbs1:  cpi r16, '1'                      ;see if second char is '1'
        brne opbs2                        ;if not '1', then see if 2
        in r16, PORTA                     ;get the data in the PORTA register
        or r16, r17                       ;OR r16 and r17 to set the selected bit
        out PORTA, r16                    ;send out the new data to PORTA
        rjmp opbdone                      ;done outputting data
opbs2:  cpi r16, '2'                      ;see if second char is '2'
        brne opbs3                        ;if not '2', then see if 3
        in r16, PORTB                     ;get the data in the PORTB register
        or r16, r17                       ;OR r16 and r17 to set the selected bit
        out PORTB, r16                    ;send out the new data to PORTB
        rjmp opbdone                      ;done outputting data
opbs3:  cpi r16, '3'                      ;see if second char is '3'
        brne opbs4                        ;if not '3', then see if ...
        in r16, PORTC                     ;get the data in the PORTC register
        or r16, r17                       ;OR r16 and r17 to set the selected bit
        out PORTC, r16                    ;send out the new data to PORTC
        rjmp opbdone
opbs4:  rcall error                       ;error condition
        rjmp opbdone                      ;ready to go back after error condition
	
opbdone:
        pop r17
        ret
				
		
;==========================================================
;Set a byte event for a change on the port
;  Command format:  V1
;==========================================================
event:
        ldd r16, Z+1                      ;get the second character (port number)
ev1:    cpi r16,'1'                       ;see if we're setting event for port 1
        brne ev2                          ;if not, then check for port 2
        in r16, ADCSR                     ;get ADC status register
        bst r16, 7                        ;get ADEN bit (ADC enable)
        brts evdone                       ;if bit set (ADC enabled), then don't allow events
        in P1_DATA, PINA                  ;if port 1, then read data on Port 1 and store in P1_DATA
        ldi r16, 255
        mov P1_BYTE_MASK, r16             ;put 255 in P1_BYTE_MASK to enable byte event on Port 1
        ldi P1_BIT_MASK, 0                ;disable bit events on port 1
        rjmp evdone
ev2:    cpi r16,'2'                       ;see if we're setting event for port 2
        brne ev3                          ;if not, then check for port 3
        in P2_DATA, PINB                  ;if port 2, then read data on Port 2 and store in P2_DATA
        ldi r16, $240
        mov P2_BYTE_MASK, r16             ;put 255 in P2_BYTE_MASK to indicate byte event on Port 2
        ldi P2_BIT_MASK, 0                ;disable bit events on port 2
        rjmp evdone
ev3:    cpi r16,'3'                       ;see if we're setting event for port 3
        brne evD                          ;if not, then check for port 4
        in P3_DATA, PINC                  ;if port 3, then read data on Port 3 and store in P3_DATA
        ldi r16, 255
        mov P3_BYTE_MASK, r16             ;put 255 in P3_BYTE_MASK to indicate byte event on Port 3
        ldi P3_BIT_MASK, 0                ;disable bit events on port 3
        rjmp evdone
evD:    rcall error                       ;else, error condition	
        rjmp evdone

evdone: ret

;==========================================================
;Set an event for a change on a bit of the port
;unless byte events are enabled for that port
; OR,in the case of port 1, don't enable events if in analog mode
;   Command format:  v1.7
;==========================================================
eventbit:
        push r17                          ;save r17
		
        ldd r16, Z+3                      ;get the bit number in r16
        subi r16, 48                      ;convert the ASCII number to binary (o to 7), store in r16
        ldi r17, 1                        ;put 1 in r17
evsh:   cpi r16, 0                        ;see if r16 is 0 yet
        breq evshd                        ;if so, then we're done shifting
        lsl r17                           ;logical shift left on r17
        dec r16                           ;decrement r16
        rjmp evsh                         ;go back until r16=0	
evshd:                                   	;r17 now contains a 1 in the place of the bit to be cleared or set

        ldd r16, Z+1                      ;get the second character (port number)
evb1:   cpi r16, '1'                      ;see if we're dealing with port 1 
        brne evb2                         ;if not, see if we're dealing with port 2
        in r16, ADCSR                     ;get ADC status register
        bst r16, 7                        ;get ADEN bit (ADC enable)
        brts evbdon                       ;if bit set (ADC enabled), then don't allow events
        mov r16, P1_BYTE_MASK             ;port 1's byte event enabled indicator
        cpi r16, 255                      ;see if byte events are enabled on port 1
        breq evbdon                       ;if byte events enabled, then don't enable bit ev.
        or P1_BIT_MASK, r17               ;if Port 1, then set the appropriate bit in the bit event mask  
        in P1_DATA, PINA                  ;and read in the data on Port 1, storing in event data reg. P1_DATA
        rjmp evbdon

evb2:   cpi r16, '2'                      ;see if we're dealing with port 2
        brne evb3                         ;if not, see if we're dealing with port 3
        mov r16, P2_BYTE_MASK             ;port 2's byte event enabled indicator
        cpi r16, 255                      ;see if byte events are enabled on port 2
        breq evbdon                       ;if byte events enabled, then don't enable bit ev.
        or P2_BIT_MASK, r17               ;if Port 2, then set the appropriate bit in the bit event mask  
        in P2_DATA, PINB                  ;and read in the data on Port 2, storing in event data reg.   
        rjmp evbdon

evb3:   cpi r16, '3'                      ;see if we're dealing with port 3
        brne evb4                         ;if not, then there's a problem at this point
        mov r16, P3_BYTE_MASK             ;port 3's byte event enabled indicator
        cpi r16, 255                      ;see if byte events are enabled on port 3
        breq evbdon                       ;if byte events enabled, then don't enable bit ev.
        or P3_BIT_MASK, r17               ;if Port 3, then set the appropriate bit in the bit event mask r26
        in P3_DATA, PINC                  ;and read in the data on Port 3, storing in event data reg.   
        rjmp evbdon

evb4:   rcall error                       ;report the error condition
        rjmp evbdon

evbdon: pop r17                           ;restore r17
        ret

;==========================================================
; Clear byte events on a port
;   Command format:  C2
;==========================================================
clrev:                                    ;clear byte events
        ldd r16, Z+1                      ;get the second character (port number)
cev1:   cpi r16,'1'                       ;see if we're setting event for port 1
        brne cev2                         ;if not, then check for port 2
        ldi r16, 0                        ;load 0 into r16
        mov P1_BYTE_MASK, r16             ;put 0 in P1_BYTE_MASK to indicate no byte event on Port 1
        rjmp cevdone
cev2:   cpi r16,'2'                       ;see if we're setting event for port 2
        brne cev3                         ;if not, then check for port 3
        ldi r16, 0                        ;put 0 in r16
        mov P2_BYTE_MASK, r16             ;put 0 in P2_BYTE_MASK to indicate no byte event on Port 2
        rjmp cevdone
cev3:   cpi r16,'3'                       ;see if we're setting event for port 3
        brne cev4                         ;if not, then there's a problem
        ldi r16, 0                        ;put 0 in r16
        mov P3_BYTE_MASK, r16             ;put 0 in P3_BYTE_MASK to indicate no event on Port 3
        rjmp cevdone
cev4:   rcall error                       ;else, error condition	
        rjmp cevdone

cevdone:
        ret


;==========================================================
;clear an event for a change on a bit of the port
;  Command format:  c2.5
;==========================================================
clrevbit:
        push r17                          ;save r17
		
        ldd r16, Z+3                      ;get the bit number in r16
        subi r16, 48                      ;convert the ASCII number to binary (o to 7), store in r16
        ldi r17, 1                        ;put 1 in r17
cevsh:  cpi r16, 0                        ;see if r16 is 0 yet
        breq cevshd                       ;if so, then we're done shifting
        lsl r17                           ;logical shift left on r17
        dec r16                           ;decrement r16
        rjmp cevsh                        ;go back until r16=0	
cevshd:                     ;r17 now contains a 1 in the place of the bit to be cleared
        com r17                           ;invert r17 so it contains 0 in the place of bit to be cleared

        ldd r16, Z+1                      ;get the second character (port number)
cevb1:  cpi r16, '1'                      ;see if we're dealing with port 1
        brne cevb2                        ;if not, see if we're dealing with port 2
        and P1_BIT_MASK, r17              ;if Port 1, then clear the appropriate bit in the bit event mask  
        rjmp cevbdon

cevb2:  cpi r16, '2'                      ;see if we're dealing with port 2
        brne cevb3                        ;if not, see if we're dealing with port 3
        and P2_BIT_MASK, r17              ;if Port 2, then clear the appropriate bit in the bit event mask  
        rjmp cevbdon

cevb3:  cpi r16, '3'                      ;see if we're dealing with port 3
        brne cevb4                        ;if not, then there's a problem
        and P3_BIT_MASK, r17              ;if Port 3, then clear the appropriate bit in the bit event mask r26
        rjmp cevbdon

cevb4:  rcall error                       ;report error condition
        rjmp cevbdon

cevbdon:
        pop r17	
        ret


;===========================================================
; Byte Event occurred on Port 1.
; New byte data is in r16.
;===========================================================
ByteEv1:                                  ;we're here for byte events on port 1
        ldi r29, 'B'                      ;if they are all set, then load 'B' for byte event
        rcall xmit                        ;and send out serial port
        ldi r29, '1'                      ;load '1' for port 1
        rcall xmit                        ;and send out serial port
        ldi r29, '='                      ;load '='
        rcall xmit                        ;and send out serial port
        rcall senddata                    ;send data in r16

        ldi r29, 13                       ;load CR and
        rcall xmit                        ;send out serial port
        ret

;===============================================================
; BitEv1 is called when a bit event has been detected on Port 1.
; When this is called, r18=new data and r17=old data.
; P1_DATA indicates which event-enabled bit(s) have changed.
;===============================================================
BitEv1:   
        push r19                         
        ldi r16, 1                        ;load r16 with 1, used for a bit-test mask
        ldi r19, 0                        ;bit number
evabk:  mov r29, r16                      ;transfer the bit-test mask into r29
        and r29, P1_DATA                  ;AND the critical info reg. with the mask
        cpi r29, 0                        ;see if the result is 0 (bit not set)
        brne evafnd                       ;if the bit is set, then jmp to evafnd
        inc r19                           ;else, increment the bit number
        lsl r16                           ;and shift the bit in the mask
        rjmp evabk                        ;and jmp back

evafnd: ldi r29, 48                       ;put 48 into r29
        add r19, r29                      ;convert bit number to ascii
        eor r17, r16                      ;modified event data to be stored
        
        ldi r29, 'b'
        rcall xmit
        ldi r29, '1'	
        rcall xmit
        ldi r29, '.'	
        rcall xmit
        mov r29, r19                      ;move the bit number into r29
        rcall xmit                        ;and send it on its way
        ldi r29, '='
        rcall xmit
        and r16, r17    
        cpi r16, 0
        breq abitzero
        ldi r29, '1'
        rjmp aaaa
abitzero:
        ldi r29, '0'
aaaa:   rcall xmit

        ldi r29, 13
        rcall xmit
eventadun:
        pop r19
        ret

;===========================================================
; Byte Event occurred on Port 2.
; New byte data is in r16.
;===========================================================
ByteEv2:
        ldi r29, 'B'                      ;if they are all set, then load 'B' for byte event
        rcall xmit                        ;and send out serial port
        ldi r29, '2'                      ;load '2' for port 2
        rcall xmit                        ;and send out serial port
        ldi r29, '='                      ;load '='
        rcall xmit                        ;and send out serial port
        rcall senddata                    ;send data in r16
        ldi r29, 13                       ;load CR and
        rcall xmit                        ;send out serial port
        ret

;===============================================================
; BitEv2 is called when a bit event has been detected on Port 2.
; When this is called, r18=new data and r17=old data.
; P2_DATA indicates which event-enabled bit(s) have changed.
;===============================================================
BitEv2: 
        push r19
        ldi r16, 1                        ;load r16 with 1, used for a bit-test mask
        ldi r19, 0                        ;bit number
evbbk:  mov r29, r16                      ;transfer the bit-test mask into r29
        and r29, P2_DATA                  ;AND the critical info reg. with the mask
        cpi r29, 0                        ;see if the result is 0 (bit not set)
        brne evbfnd                       ;if the bit is set, then jmp to evafnd
        inc r19                           ;else, increment the bit number
        lsl r16                           ;and shift the bit in the mask
        rjmp evbbk                        ;and jmp back

evbfnd: ldi r29, 48                       ;put 48 into r29
        add r19, r29                      ;convert bit number to ascii
        eor r17, r16

        ldi r29, 'b'
        rcall xmit
        ldi r29, '2'	
        rcall xmit
        ldi r29, '.'	
        rcall xmit
        mov r29, r19                      ;put bit number into r29
        rcall xmit
        ldi r29, '='
        rcall xmit
        and r16, r17
        cpi r16, 0
        breq bbitzero
        ldi r29, '1'
        rjmp bbbb
bbitzero:
        ldi r29, '0'
bbbb:   rcall xmit

        ldi r29, 13
        rcall xmit
eventbdun:
        pop r19
        ret

;===========================================================
; Byte Event occurred on Port 3.
; New byte data is in r16.
;===========================================================
ByteEv3:
        ldi r29, 'B'                      ;if they are all set, then load 'B' for byte event
        rcall xmit                        ;and send out serial port
        ldi r29, '3'                      ;load '3' for port 3
        rcall xmit                        ;and send out serial port
        ldi r29, '='                      ;load '='
        rcall xmit                        ;and send out serial port
        rcall senddata                    ;send data in r16
        ldi r29, 13                       ;load CR and
        rcall xmit                        ;send out serial port
        ret

;===============================================================
; BitEv3 is called when a bit event has been detected on Port 3.
; When this is called, r18=new data and r17=old data.
; P3_DATA indicates which event-enabled bit(s) have changed.
;===============================================================
BitEv3:                 ;sub for bit events on 3, P3_DATA contains critical data
        push r19

        ldi r16, 1                        ;load r16 with 1, used for a bit-test mask
        ldi r19, 0                        ;bit number
evcbk:  mov r29, r16                      ;transfer the bit-test mask into r29
        and r29, P3_DATA                  ;AND the critical info reg. with the mask
        cpi r29, 0                        ;see if the result is 0 (bit not set)
        brne evcfnd                       ;if the bit is set, then jmp to evafnd
        inc r19                           ;else, increment the bit number
        lsl r16                           ;and shift the bit in the mask
        rjmp evcbk                        ;and jmp back

evcfnd: ldi r29, 48                       ;put 48 into r29
        add r19, r29                      ;convert bit number to ascii
        eor r17, r16

        ldi r29, 'b'
        rcall xmit
        ldi r29, '3'	
        rcall xmit
        ldi r29, '.'	
        rcall xmit
        mov r29, r19                      ;put bit number into r29
        rcall xmit                        ;and send on its way
        ldi r29, '='
        rcall xmit
        and r16, r17    ;
        cpi r16, 0
        breq cbitzero
        ldi r29, '1'
        rjmp cccc
cbitzero:
        ldi r29, '0'
cccc:   rcall xmit

        ldi r29, 13
        rcall xmit
eventcdun:
        pop r19
        ret

;=====================================================
; Enable the buffer for a port
;   Command format:  E2
;=====================================================
enport:                 
        ldd r16, Z+1                      ;get the second character
en1:    cpi r16, '1'                      ;see if we're trying to enable port 1
        brne en2                          ;if not, then see about port 2
        in r16, PORTD                     ;get PORTD data
        cbr r16, 4                        ;clear bit 2, port 1 enable
        out PORTD, r16                    ;send out new value, enabling port 1
        rjmp enpdone

en2:    cpi r16, '2'                      ;see if we're tyring to enable port 2
        brne en3                          ;ifnot, then see about port 3
        in r16, PORTD                     ;get PORTD data
        cbr r16, $10                      ;clear bit 4, port 2 enable 
        out PORTD, r16                    ;send out new value, enabling port 2
        rjmp enpdone	

en3:    cpi r16, '3'                      ;see if we're tyring to enable port 3
        brne en4                          ;if not, then continue..
        in r16, PORTD                     ;get PORTD data
        cbr r16, $40                      ;clear bit 6, port 3 enable
        out PORTD, r16                    ;send out new value, enable port 3
        rjmp enpdone

en4:    rcall error

enpdone:
        ret

;===================================================
; Disable the buffer for a port
;   Command format:  X2
;===================================================
disport:               	;used to disable a port
        ldd r16, Z+1                      ;get the second character
dis1:   cpi r16, '1'                      ;see if we're trying to disable port 1
        brne dis2                         ;if not, then see about port 2
        in r16, PORTD                     ;get PORTD data
        sbr r16, 4                        ;set bit 2, port 1 disable
        out PORTD, r16                    ;send out new value, disabling port 1
        rjmp dispdone

dis2:   cpi r16, '2'                      ;see if we're tyring to disable port 2
        brne dis3                         ;if not, then see about port 3
        in r16, PORTD                     ;get PORTD data
        sbr r16, $10                      ;set bit 4, port 2 disable 
        out PORTD, r16                    ;send out new value, disabling port 2
        rjmp dispdone	

dis3:   cpi r16, '3'                      ;see if we're tyring to disable port 3
        brne dis4                         ;if not, then continue..
        in r16, PORTD                     ;get PORTD data
        sbr r16, $40                      ;set bit 6, port 3 disable
        out PORTD, r16                    ;send out new value, disable port 3
        rjmp dispdone

dis4:   rcall error

dispdone:
        ret

;===============================================================
; Ping can be used to test the serial connection.
; It sends the character 'G' and a carriage return.
;===============================================================
ping:   ldi r29, 'G'
        rcall xmit
        ldi r29, 13
        rcall xmit
        ret

;===============================================================
; Set up Port 1 for Analog Input operation.
;===============================================================
SetAnalog:
        push r17
        ldi r17, 0              
        out DDRA, r17                     ;put port 1 in input mode
        out ADCSR, r17                    ;clear ADC control reg

        sbi ADCSR, 7                      ;set ADEN, enables ADC unit
        sbi ADCSR, 6                      ;set ADSC to initiate a conv.
        sbi ADCSR, 1                    ;set prescaler bit
        sbi ADCSR, 0                    ;set prescaler bit

        nop
        nop
        nop

adw:    sbis ADCSR, 4                     ;skip next instruction if ADIF set
        rjmp adw                          ;wait for the conversion to end

        sbi ADCSR, 4                    ; this clears the bit
        in r17, ADCL
        in r17, ADCH

        ldi P1_BIT_MASK, 0                ;disable port 1 bit events
        mov P1_BYTE_MASK, P1_BIT_MASK     ;disable port 1 byte events
        pop r17
        ret

;===============================================================
; Set up Port 1 for digital I/O operation.
;===============================================================
SetDig:
        cbi ADCSR, 7                      ;clear ADEN, disable ADC conv.        
        ret

;=====================================get analog sub
GetAnalog:                      ;select proper channel, do a conversion,
                                ;and return the result
        push r17                          ;save r17

        ldd r17, Z+2                      ;get the ASCII value of channel
        push r17                          ;save the channel
        subi r17, 48                      ;convert ASCII to a number
        out ADMUX, r17                    ;select the desired channel
        nop
        nop
        nop
        nop
        nop                               ;wait a bit
        nop
        nop
        nop
        nop
        nop
        nop
        sbi ADCSR, 6                      ;set ADSC to initiate a conv.

adw2:
        ;sbic ADCSR, 6                     ;skip next instruction if ADSC cleared
        sbis ADCSR, 4                     ;skip next instr if ADIF set,
                                          ;meaning registers have been updated

        rjmp adw2                         ;wait for the conversion to end

        sbi ADCSR, 4                      ;clear the bit

        in r16, ADCL                      ;get lower 8 bits
        in r17, ADCH                      ;get upper 2 bits

        ldi r29, 'a'                      ;load 'a' in r29
        rcall xmit                        ;and send it out
        ldi r29, '.'                      ;load '.' in r29
        rcall xmit                        ;and send it out
        pop r29                           ;retrieve channel #
        rcall xmit                        ;and send it
        ldi r29, '='                      ;load '=' in r29
        rcall xmit                        ;and send it
        ldi r29, 48                       ;put 48 in r29
        add r29, r17                      ;convert highest char to ASCII
        rcall xmit                        ;and send on its way
        rcall senddata                    ;send value in analog data r16
        ldi r29, 13
        rcall xmit                        ;send CR
        pop r17                           ;retrieve r17
        ret

;===============================================================
; Convert nibble (0 to 15) in r17 to a hex character
; and return in r29.
;===============================================================
ConvR17:
        cpi r17, 10                       ;compare the character to 10
        brsh abcdef                       ;if r17 is 10 or greater, then branch to abcdef
        ldi r29, 48                       ;otherwise, put 48 in r29
        add r29, r17                      ;add 48 to the number (0 - 9) to convert to ASCII
        rjmp ConvR17done
abcdef: ldi r29, 55                       ;put 55 in r29
        add r29, r17                      ;add 55 to the number (10 - 15) to convert to ASCII

ConvR17done:
        ret                               ;go back from whence we came

;================================================================
; Convert the character in r19 to a binary number from 0 to 15.
; and return in r19.  On error, it returns 255 in r19.
;================================================================
getdata:
        cpi r19, 48                       ;compare data character with 48 (0)
        brlo getderror                    ;if lower than 48, error condition
        cpi r19, 58                       ;compare data character with 58 (:)
        brsh getdchkAF                    ;if higher than 57 (9), then see if it's A - F
        subi r19, 48                      ;else, it's from 48 to 57.. convert to binary
        rjmp getdgobak                    ;all done
getdchkAF:
        cpi r19, 65                       ;compare character with 65 (A)
        brlo getderror                    ;if LT 65 and GT 57, we've got an error
        cpi r19, 71                       ;else, compare with 71 (G)
        brsh getdchkaf2                   ;if GT 70 (F), then see if it's a - f
        subi r19, 55                      ;else, it's from A - F... convert to binary
        rjmp getdgobak                    ;all done
getdchkaf2:
        cpi r19, 'a'                      ;compare character with 'a' (97)
        brlo getderror                    ;if LT 'a' and GT 'F', we've got an error
        cpi r19, 'g'                      ;else, compare with 'g'
        brsh getderror                    ;if GT 'f' then it's an error
        subi r19, 87                      ;else, it's from a-f... convert to binary
        rjmp getdgobak                    ;all done, go back with number in r19
getderror:
        ldi r19, 255                      ;put error condition data in r19
getdgobak:
        ret

;============================================================
; XMIT transmits whatever is in r29 over the serial connection.
; If r29 contains 13 (CR), then 10 (LF) is also sent after it.
;============================================================	
xmit:   push r19
xmit2:  in r19, USR                       ;wait until ready to transmit
        andi r19, $20
        breq xmit2
        out UDR, r29
        
        cpi r29, 13                       ;see if we just sent a CR
        brne xmitdone                     ;if not, then we're done
        mov r19, LF_FLAG                  ;get the LF indicator
        cpi r19, 0                        ;and see if it's 0
        breq xmitdone                     ;if so (flag=0), then don't xmit a LF
        ldi r29, 10                       ;else, load the LF character
xmit2LF:
        in r19, USR                       ;wait until ready to transmit
        andi r19, $20
        breq xmit2LF
        out UDR, r29
        
xmitdone:
        pop r19
        ret

;===================================================================
; Send out in ASCII hexadecimal the number it receives in r16
;===================================================================
senddata:
        push r17
        mov r17, r16                      ;store the data
        andi r17, 240                     ;clear the lower nibble
        swap r17                          ;put high nibble in low nibble
        cpi r17, 10                       ;compare data with 10
        brsh sdadd55                      ;if greater than 9, it's A-F
        ldi r29, 48                       ;else, its 0 - 9
        add r29, r17                      ;convert to ASCII
        rcall xmit                        ;and send on its way
        rjmp sd2nd                        ;and move on to the next char
sdadd55:
        ldi r29, 55                       ;it's A-F
        add r29, r17                      ;convert to ASCII
        rcall xmit                        ;and send on its way

sd2nd:  andi r16, 15                      ;clear the upper nibble
        cpi r16, 10                       ;compare data with 10
        brsh sdda55                       ;if greater than 9, it's A-F
        ldi r29, 48                       ;else, it's 0-9
        add r29, r16                      ;convert to ASCII
        rcall xmit                        ;and send on its way
        rjmp sddone                       ;done sending
sdda55: ldi r29, 55                       ;it's A-F
        add r29, r16                      ;convert to ASCII
        rcall xmit                        ;and send on its way
        
sddone: pop r17
        ret

;==========================================================
; UART_Setup selects the desired baud rate and enables UART
; assuming an 8MHZ crystal frequency is being used.
;==========================================================
UART_Setup:
        in r18, PIND                      ;get data on port D pins
        andi r18, $28                     ;00101000, isolate just the two bits


        ldi r16, $FC                      ;252 = 11111100
        out DDRD, r16                     ;set up the upper 6 bits of port D for outputs

        ldi r16, 0
        out PORTD, r16                    ;ports 1 - 3 are enabled as inputs

baud1:  cpi r18, 0                        ;00000000 = 0, baud rate 1
        brne baud2                        ;if not baud rate 1, then check next
        ldi r16, 207                      ;UBRR setting for 2400 baud
        rjmp ldubrr                       ;skip the rest of the options

baud2:  cpi r18, 8                        ;00001000 = 8, baud rate 2
        brne baud3                        ;if not baud rate 2, then check next
        ldi r16, 103                      ;UBRR setting for 4800 baud
        rjmp ldubrr                       ;skip the rest of the options

baud3:  cpi r18, $20                      ;00100000 = $20, baud rate 3
        brne baud4                        ;if not baud rate 3, then check next
        ldi r16, 51                       ;UBRR setting for 9600 baud
        rjmp ldubrr                       ;skip the rest of the options

baud4:  cpi r18, $28                      ;00101000 = $28, baud rate 4
        brne baud5                        ;if not baud rate 4, then check next
        ldi r16, 25                       ;UBRR setting for 19200 baud
        rjmp ldubrr                       ;skip the rest of the options

baud5:  ldi r16, 51                       ;default to baud rate 9600 on error

ldubrr: out UBRR, r16                     ;set baud rate to desired

        ret

;===========================================================
; FIND_NEXT_CMD  -- upon return, Z points to first character
; past the CR that ends the current command.
;===========================================================
FIND_NEXT_CMD:
        push r16
keep_looking:
        ld r16, Z+                        ;put character in r16 and point to next char.
        cpi r16, 13                       ;is it a carriage return?
        breq Found_Next                   ;if so, then we're done
        rjmp keep_looking                 ;else, loop back until it's found
Found_Next:
        ldi r16, BUFFER_START+BUFFER_LENGTH
        cp ZL, r16                        ;see we've gone outside the buffer
        brlo No_Rollover_Z                ;if ZL<end_of_buffer, then no rollover
        ldi ZL, BUFFER_START              ;else, go back to the start
No_Rollover_Z:

        pop r16
        ret                               ;go back from whence we came


;===========================================================
; ToggleLF changes the LF_FLAG
;===========================================================
ToggleLF:
        com LF_FLAG                       ;replace with the one's complement
        ret

